<?php

/*
 * Copyright (C) 2008 - Till Glggler     <tgloeggl@uos.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */


/**
 * @author    tgloeggl
 * @copyright (c) Authors
 * @version   $Id$
 */

define('MAX_ROWS', 14);
define('COLUMNS', 5);

class SchlaumeierPlugin extends AbstractStudIPStandardPlugin {

	private $colors = array('red', 'green', 'blue', 'yellow', 'black', 'white', 'pink', 'lightblue');
	private $no_doubles = true;

  function SchlaumeierPlugin() {

    parent::AbstractStudIPStandardPlugin();

    $this->setPluginiconname("img/plugin.png");

    # navigation
    $navigation =& new PluginNavigation();
    $navigation->setDisplayname(_("Schlaumeier"));
    $this->setNavigation($navigation);

    ## AbstractStudIPStandardPlugin specifics
	}


	function initialize() {
		global $_include_additional_header, $sess;

		// we want to display the dates in german
		setlocale(LC_TIME, 'de_DE@euro', 'de_DE', 'de', 'ge');

		// the default for displaying timestamps
		$this->time_format_string = "%A %d. %B %Y, %H:%M";
		$this->time_format_string_short = "%a %d. %B %Y, %H:%M";

    $this->template_factory =
	    new Flexi_TemplateFactory(dirname(__FILE__).'/templates');

		// path to plugin-pictures
		$this->picturepath =  $GLOBALS['CANONICAL_RELATIVE_PATH_STUDIP'] . $this->getPluginPath() . '/img';

		$_include_additional_header = 
			'<link rel="stylesheet" href="'. PluginEngine::getLink($this, array(), 'css') .'" type="text/css">'; 

		$sess->register('schlaumeier_game_data');
	}

	function getPluginname() {
		return _("Schlaumeier");
	}

	function getDisplaytitle() {
		return $this->getPluginname();
	}

	function actionBackend() {
		global $schlaumeier_game_data, $_REQUEST;

		// this hack is necessary to disable the standard Stud.IP layout
    ob_end_clean();

		switch ($_REQUEST['cmd']) {
			case 'check':
				$correct = 0;
				$near = 0;
				$solution_fields = $schlaumeier_game_data['solution'];
				$request_fields = array();
				for ($i = 0; $i < COLUMNS; $i++) {
					$request_fields[] = $_REQUEST['field_' . $i];
				}

				foreach ($solution_fields as $field => $col_num)  {
					if ($request_fields[$field] == $this->colors[$col_num]) {
						$correct++;
						unset($request_fields[$field]);
						unset($solution_fields[$field]);
					}
				}

				foreach ($solution_fields as $field_pos => $col_num) {
					foreach ($request_fields as $req_pos => $color) {
						if ($req_pos != $field_pos) {
							if ($this->colors[$col_num] == $color) {
								$near++;
								unset($request_fields[$req_pos]);
								break;
							}
						}
					}
				}

				echo $correct .'|'. $near;
				break;

			case 'solution':
				echo '<font size="6">L&ouml;sung:</font><br/>';
				foreach ($schlaumeier_game_data['solution'] as $col_num)  {
					echo '<img src="'. $this->getPluginUrl() .'/images/'. $this->colors[$col_num] . '.png">&nbsp;';
				}
				echo '<hr/>';
				break;

			case 'new_game':
				//new_game();
				break;

		}
		die;
	}

	function actionNewGame() {
		$this->startNewGame();
		$this->actionShow();
	}

  function actionShow() {
		global $sess, $schlaumeier_game_data;

		if (!$schlaumeier_game_data) {
			$this->startNewGame();
		}

		$template =& $this->template_factory->open('main_view');
		$template->set_layout('layout.php');

		// set the correct paths
		$template->set_attribute('image_path', $this->getPluginUrl() . '/images/');
		$template->set_attribute('backend_path', PluginEngine::getLink($this, array(), 'backend'));
		$template->set_attribute('new_game_path', PluginEngine::getLink($this, array(), 'newgame'));

		$template->set_attribute('colors', $this->colors);
		$template->set_attribute('schlaumeier_game_data', $schlaumeier_game_data);
		$template->set_attribute('no_doubles', $this->no_doubles);

		echo $template->render();
  }


	function setPluginPath($newpath) {
		parent::setPluginPath($newpath);
		//$this->buildMenu();
	}

	function buildMenu() {
		$navigation = new PluginNavigation();
		$navigation->setDisplayname('Forum++');
		$navigation->addLinkParam('source', 'va');
		$navigation->setActive();

		
		if ($this->rechte) {
			$sub_nav = new PluginNavigation();
			$sub_nav->setDisplayname(_("Konfiguration"));
			$sub_nav->addLinkParam('plugin_subnavi_params', 'config');
			$navigation->addSubmenu($sub_nav);
		}

		$this->setNavigation($navigation);
	}

  function actionShowAdministrationPage() {
    printf("in %s:%s\n", __CLASS__, __FUNCTION__);
  }


  // AbstractStudIPStandardPlugin specifics
	function getScore() {
		return 10;
	}

	/**
	 * alternative makebutton-function, to enable use of buttons which are not included in Stud.IP
	 * @param string $name name of the button
	 * @param string $type one of full / src. full returns the button-image with button-tags surrounded
	 * @returns string
	 */
	function makebutton($name, $type = 'full') {
		$img = $this->getPluginPath() .'/buttons/'. $GLOBALS['_language_path'] .'/'. $name .'-button.png'; 
		switch ($type) {
			case 'src':
				return ' src="'. $img .'" ';
				break;

			case ' full':
			default:
				return '<button><img src="'. $img .'"></button>';
				break;
		}
	}

	function startNewGame() {
		global $sess, $schlaumeier_game_data;

		$schlaumeier_game_data['solution'] = array();
		srand();

		$used = array();
		for ($i = 0; $i < COLUMNS; $i++) {
			do {
				$col = rand(0, sizeof($this->colors) - 1);
			} while ($used[$col] == true);

			if ($this->no_doubles) {
				$used[$col] = true;
			}

			$schlaumeier_game_data['solution'][] = $col;
		}
	}

}
